@@ -106,6 +106,7 @@ typedef struct r_reg_set_t {
 	RRegArena *arena;
 	RList *pool;      /* RRegArena */
 	RList *regs;      /* RRegItem */
+	HtPP *ht_regs;    /* name:RRegItem */
 	RListIter *cur;
 	int maskregstype; /* which type of regs have this reg set (logic mask with RRegisterType  R_REG_TYPE_XXX) */
 } RRegSet;
@@ -117,6 +118,7 @@ typedef struct r_reg_t {
 	char *name[R_REG_NAME_LAST]; // aliases
 	RRegSet regset[R_REG_TYPE_LAST];
 	RList *allregs;
+	RList *roregs;
 	int iters;
 	int arch;
 	int bits;
@@ -138,10 +140,11 @@ typedef struct r_reg_flags_t {
 R_API void r_reg_free(RReg *reg);
 R_API void r_reg_free_internal(RReg *reg, bool init);
 R_API RReg *r_reg_new(void);
-R_API int r_reg_set_name(RReg *reg, int role, const char *name);
-R_API int r_reg_set_profile_string(RReg *reg, const char *profile);
-R_API int r_reg_set_profile(RReg *reg, const char *profile);
-R_API int r_reg_parse_gdb_profile(const char *profile);
+R_API bool r_reg_set_name(RReg *reg, int role, const char *name);
+R_API bool r_reg_set_profile_string(RReg *reg, const char *profile);
+R_API bool r_reg_set_profile(RReg *reg, const char *profile);
+R_API bool r_reg_parse_gdb_profile(const char *profile);
+R_API bool r_reg_is_readonly(RReg *reg, RRegItem *item);
 
 R_API RRegSet *r_reg_regset_get(RReg *r, int type);
 R_API ut64 r_reg_getv(RReg *reg, const char *name);
@@ -167,7 +170,10 @@ R_API int r_reg_type_by_name(const char *str);
 R_API int r_reg_get_name_idx(const char *type);
 
 R_API RRegItem *r_reg_cond_get(RReg *reg, const char *name);
+R_API void r_reg_cond_apply(RReg *r, RRegFlags *f);
+R_API bool r_reg_cond_set(RReg *reg, const char *name, bool val);
 R_API int r_reg_cond_get_value(RReg *r, const char *name);
+R_API bool r_reg_cond_bits_set(RReg *r, int type, RRegFlags *f, bool v);
 R_API int r_reg_cond_bits(RReg *r, int type, RRegFlags *f);
 R_API RRegFlags *r_reg_cond_retrieve(RReg *r, RRegFlags *);
 R_API int r_reg_cond(RReg *r, int type);
